@@ -64,13 +64,15 @@ static ULONG STDMETHODCALLTYPE d3d11_blend_state_AddRef(ID3D11BlendState *iface)
 
     TRACE("%p increasing refcount to %u.\n", state, refcount);
 
-    return refcount;
-}
+    if (refcount == 1)
+    {
+        ID3D11Device2_AddRef(state->device);
+        wined3d_mutex_lock();
+        wined3d_blend_state_incref(state->wined3d_state);
+        wined3d_mutex_unlock();
+    }
 
-static void d3d_blend_state_cleanup(struct d3d_blend_state *state)
-{
-    wined3d_private_store_cleanup(&state->private_store);
-    ID3D11Device_Release(state->device);
+    return refcount;
 }
 
 static ULONG STDMETHODCALLTYPE d3d11_blend_state_Release(ID3D11BlendState *iface)
@@ -82,12 +84,13 @@ static ULONG STDMETHODCALLTYPE d3d11_blend_state_Release(ID3D11BlendState *iface
 
     if (!refcount)
     {
-        struct d3d_device *device = impl_from_ID3D11Device(state->device);
+        ID3D11Device2 *device = state->device;
+
         wined3d_mutex_lock();
-        wine_rb_remove(&device->blend_states, &state->entry);
-        d3d_blend_state_cleanup(state);
+        wined3d_blend_state_decref(state->wined3d_state);
         wined3d_mutex_unlock();
-        HeapFree(GetProcessHeap(), 0, state);
+
+        ID3D11Device2_Release(device);
     }
 
     return refcount;
@@ -100,7 +103,7 @@ static void STDMETHODCALLTYPE d3d11_blend_state_GetDevice(ID3D11BlendState *ifac
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -203,7 +206,7 @@ static void STDMETHODCALLTYPE d3d10_blend_state_GetDevice(ID3D10BlendState1 *ifa
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device2_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_blend_state_GetPrivateData(ID3D10BlendState1 *iface,
@@ -288,24 +291,25 @@ static const struct ID3D10BlendState1Vtbl d3d10_blend_state_vtbl =
     d3d10_blend_state_GetDesc1,
 };
 
-static HRESULT d3d_blend_state_init(struct d3d_blend_state *state, struct d3d_device *device,
-        const D3D11_BLEND_DESC *desc)
+static void STDMETHODCALLTYPE d3d_blend_state_wined3d_object_destroyed(void *parent)
 {
-    state->ID3D11BlendState_iface.lpVtbl = &d3d11_blend_state_vtbl;
-    state->ID3D10BlendState1_iface.lpVtbl = &d3d10_blend_state_vtbl;
-    state->refcount = 1;
-    wined3d_private_store_init(&state->private_store);
-    state->desc = *desc;
+    struct d3d_blend_state *state = parent;
+    struct d3d_device *device = impl_from_ID3D11Device2(state->device);
 
-    state->device = &device->ID3D11Device_iface;
-    ID3D11Device_AddRef(state->device);
-
-    return S_OK;
+    wine_rb_remove(&device->blend_states, &state->entry);
+    wined3d_private_store_cleanup(&state->private_store);
+    heap_free(parent);
 }
 
+static const struct wined3d_parent_ops d3d_blend_state_wined3d_parent_ops =
+{
+    d3d_blend_state_wined3d_object_destroyed,
+};
+
 HRESULT d3d_blend_state_create(struct d3d_device *device, const D3D11_BLEND_DESC *desc,
         struct d3d_blend_state **state)
 {
+    struct wined3d_blend_state_desc wined3d_desc;
     struct d3d_blend_state *object;
     struct wine_rb_entry *entry;
     D3D11_BLEND_DESC tmp_desc;
@@ -337,9 +341,6 @@ HRESULT d3d_blend_state_create(struct d3d_device *device, const D3D11_BLEND_DESC
                     tmp_desc.RenderTarget[i].RenderTargetWriteMask, i);
     }
 
-    /* glSampleCoverage() */
-    if (tmp_desc.AlphaToCoverageEnable)
-        FIXME("Ignoring AlphaToCoverageEnable %#x.\n", tmp_desc.AlphaToCoverageEnable);
     /* glEnableIndexedEXT(GL_BLEND, ...) */
     if (tmp_desc.IndependentBlendEnable)
         FIXME("Per-rendertarget blend not implemented.\n");
@@ -357,30 +358,45 @@ HRESULT d3d_blend_state_create(struct d3d_device *device, const D3D11_BLEND_DESC
         return S_OK;
     }
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+    if (!(object = heap_alloc_zero(sizeof(*object))))
     {
         wined3d_mutex_unlock();
         return E_OUTOFMEMORY;
     }
 
-    if (FAILED(hr = d3d_blend_state_init(object, device, &tmp_desc)))
-    {
-        WARN("Failed to initialize blend state, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
-        wined3d_mutex_unlock();
-        return hr;
-    }
+    object->ID3D11BlendState_iface.lpVtbl = &d3d11_blend_state_vtbl;
+    object->ID3D10BlendState1_iface.lpVtbl = &d3d10_blend_state_vtbl;
+    object->refcount = 1;
+    wined3d_private_store_init(&object->private_store);
+    object->desc = tmp_desc;
 
-    if (wine_rb_put(&device->blend_states, desc, &object->entry) == -1)
+    if (wine_rb_put(&device->blend_states, &tmp_desc, &object->entry) == -1)
     {
         ERR("Failed to insert blend state entry.\n");
-        d3d_blend_state_cleanup(object);
-        HeapFree(GetProcessHeap(), 0, object);
+        wined3d_private_store_cleanup(&object->private_store);
+        heap_free(object);
         wined3d_mutex_unlock();
         return E_FAIL;
     }
+
+    wined3d_desc.alpha_to_coverage = desc->AlphaToCoverageEnable;
+
+    /* We cannot fail after creating a wined3d_blend_state object. It
+     * would lead to double free. */
+    if (FAILED(hr = wined3d_blend_state_create(device->wined3d_device, &wined3d_desc,
+            object, &d3d_blend_state_wined3d_parent_ops, &object->wined3d_state)))
+    {
+        WARN("Failed to create wined3d blend state, hr %#x.\n", hr);
+        wined3d_private_store_cleanup(&object->private_store);
+        wine_rb_remove(&device->blend_states, &object->entry);
+        heap_free(object);
+        wined3d_mutex_unlock();
+        return hr;
+    }
     wined3d_mutex_unlock();
 
+    ID3D11Device2_AddRef(object->device = &device->ID3D11Device2_iface);
+
     TRACE("Created blend state %p.\n", object);
     *state = object;
 
@@ -450,7 +466,7 @@ static ULONG STDMETHODCALLTYPE d3d11_depthstencil_state_AddRef(ID3D11DepthStenci
 static void d3d_depthstencil_state_cleanup(struct d3d_depthstencil_state *state)
 {
     wined3d_private_store_cleanup(&state->private_store);
-    ID3D11Device_Release(state->device);
+    ID3D11Device2_Release(state->device);
 }
 
 static ULONG STDMETHODCALLTYPE d3d11_depthstencil_state_Release(ID3D11DepthStencilState *iface)
@@ -462,12 +478,12 @@ static ULONG STDMETHODCALLTYPE d3d11_depthstencil_state_Release(ID3D11DepthStenc
 
     if (!refcount)
     {
-        struct d3d_device *device = impl_from_ID3D11Device(state->device);
+        struct d3d_device *device = impl_from_ID3D11Device2(state->device);
         wined3d_mutex_lock();
         wine_rb_remove(&device->depthstencil_states, &state->entry);
         d3d_depthstencil_state_cleanup(state);
         wined3d_mutex_unlock();
-        HeapFree(GetProcessHeap(), 0, state);
+        heap_free(state);
     }
 
     return refcount;
@@ -480,7 +496,7 @@ static void STDMETHODCALLTYPE d3d11_depthstencil_state_GetDevice(ID3D11DepthSten
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -584,7 +600,7 @@ static void STDMETHODCALLTYPE d3d10_depthstencil_state_GetDevice(ID3D10DepthSten
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device2_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_depthstencil_state_GetPrivateData(ID3D10DepthStencilState *iface,
@@ -655,8 +671,7 @@ static HRESULT d3d_depthstencil_state_init(struct d3d_depthstencil_state *state,
     wined3d_private_store_init(&state->private_store);
     state->desc = *desc;
 
-    state->device = &device->ID3D11Device_iface;
-    ID3D11Device_AddRef(state->device);
+    ID3D11Device2_AddRef(state->device = &device->ID3D11Device2_iface);
 
     return S_OK;
 }
@@ -721,7 +736,7 @@ HRESULT d3d_depthstencil_state_create(struct d3d_device *device, const D3D11_DEP
         return S_OK;
     }
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+    if (!(object = heap_alloc_zero(sizeof(*object))))
     {
         wined3d_mutex_unlock();
         return E_OUTOFMEMORY;
@@ -730,7 +745,7 @@ HRESULT d3d_depthstencil_state_create(struct d3d_device *device, const D3D11_DEP
     if (FAILED(hr = d3d_depthstencil_state_init(object, device, &tmp_desc)))
     {
         WARN("Failed to initialize depthstencil state, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
+        heap_free(object);
         wined3d_mutex_unlock();
         return hr;
     }
@@ -739,7 +754,7 @@ HRESULT d3d_depthstencil_state_create(struct d3d_device *device, const D3D11_DEP
     {
         ERR("Failed to insert depthstencil state entry.\n");
         d3d_depthstencil_state_cleanup(object);
-        HeapFree(GetProcessHeap(), 0, object);
+        heap_free(object);
         wined3d_mutex_unlock();
         return E_FAIL;
     }
@@ -815,7 +830,7 @@ static ULONG STDMETHODCALLTYPE d3d11_rasterizer_state_AddRef(ID3D11RasterizerSta
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(state->device);
+        ID3D11Device2_AddRef(state->device);
         wined3d_mutex_lock();
         wined3d_rasterizer_state_incref(state->wined3d_state);
         wined3d_mutex_unlock();
@@ -833,13 +848,13 @@ static ULONG STDMETHODCALLTYPE d3d11_rasterizer_state_Release(ID3D11RasterizerSt
 
     if (!refcount)
     {
-        ID3D11Device *device = state->device;
+        ID3D11Device2 *device = state->device;
 
         wined3d_mutex_lock();
         wined3d_rasterizer_state_decref(state->wined3d_state);
         wined3d_mutex_unlock();
 
-        ID3D11Device_Release(device);
+        ID3D11Device2_Release(device);
     }
 
     return refcount;
@@ -852,7 +867,7 @@ static void STDMETHODCALLTYPE d3d11_rasterizer_state_GetDevice(ID3D11RasterizerS
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -956,7 +971,7 @@ static void STDMETHODCALLTYPE d3d10_rasterizer_state_GetDevice(ID3D10RasterizerS
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device2_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_rasterizer_state_GetPrivateData(ID3D10RasterizerState *iface,
@@ -1021,11 +1036,11 @@ static const struct ID3D10RasterizerStateVtbl d3d10_rasterizer_state_vtbl =
 static void STDMETHODCALLTYPE d3d_rasterizer_state_wined3d_object_destroyed(void *parent)
 {
     struct d3d_rasterizer_state *state = parent;
-    struct d3d_device *device = impl_from_ID3D11Device(state->device);
+    struct d3d_device *device = impl_from_ID3D11Device2(state->device);
 
     wine_rb_remove(&device->rasterizer_states, &state->entry);
     wined3d_private_store_cleanup(&state->private_store);
-    HeapFree(GetProcessHeap(), 0, parent);
+    heap_free(parent);
 }
 
 static const struct wined3d_parent_ops d3d_rasterizer_state_wined3d_parent_ops =
@@ -1053,6 +1068,7 @@ static HRESULT d3d_rasterizer_state_init(struct d3d_rasterizer_state *state, str
     }
 
     wined3d_desc.front_ccw = desc->FrontCounterClockwise;
+    wined3d_desc.depth_clip = desc->DepthClipEnable;
 
     /* We cannot fail after creating a wined3d_rasterizer_state object. It
      * would lead to double free. */
@@ -1065,7 +1081,7 @@ static HRESULT d3d_rasterizer_state_init(struct d3d_rasterizer_state *state, str
         return hr;
     }
 
-    ID3D11Device_AddRef(state->device = &device->ID3D11Device_iface);
+    ID3D11Device2_AddRef(state->device = &device->ID3D11Device2_iface);
 
     return S_OK;
 }
@@ -1093,7 +1109,7 @@ HRESULT d3d_rasterizer_state_create(struct d3d_device *device, const D3D11_RASTE
         return S_OK;
     }
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+    if (!(object = heap_alloc_zero(sizeof(*object))))
     {
         wined3d_mutex_unlock();
         return E_OUTOFMEMORY;
@@ -1104,7 +1120,7 @@ HRESULT d3d_rasterizer_state_create(struct d3d_device *device, const D3D11_RASTE
     if (FAILED(hr))
     {
         WARN("Failed to initialize rasterizer state, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
+        heap_free(object);
         return hr;
     }
 
@@ -1178,7 +1194,7 @@ static ULONG STDMETHODCALLTYPE d3d11_sampler_state_AddRef(ID3D11SamplerState *if
 
     if (refcount == 1)
     {
-        ID3D11Device_AddRef(state->device);
+        ID3D11Device2_AddRef(state->device);
         wined3d_mutex_lock();
         wined3d_sampler_incref(state->wined3d_sampler);
         wined3d_mutex_unlock();
@@ -1196,13 +1212,13 @@ static ULONG STDMETHODCALLTYPE d3d11_sampler_state_Release(ID3D11SamplerState *i
 
     if (!refcount)
     {
-        ID3D11Device *device = state->device;
+        ID3D11Device2 *device = state->device;
 
         wined3d_mutex_lock();
         wined3d_sampler_decref(state->wined3d_sampler);
         wined3d_mutex_unlock();
 
-        ID3D11Device_Release(device);
+        ID3D11Device2_Release(device);
     }
 
     return refcount;
@@ -1215,7 +1231,7 @@ static void STDMETHODCALLTYPE d3d11_sampler_state_GetDevice(ID3D11SamplerState *
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = state->device;
+    *device = (ID3D11Device *)state->device;
     ID3D11Device_AddRef(*device);
 }
 
@@ -1319,7 +1335,7 @@ static void STDMETHODCALLTYPE d3d10_sampler_state_GetDevice(ID3D10SamplerState *
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    ID3D11Device_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
+    ID3D11Device2_QueryInterface(state->device, &IID_ID3D10Device, (void **)device);
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_sampler_state_GetPrivateData(ID3D10SamplerState *iface,
@@ -1384,11 +1400,11 @@ static const struct ID3D10SamplerStateVtbl d3d10_sampler_state_vtbl =
 static void STDMETHODCALLTYPE d3d_sampler_wined3d_object_destroyed(void *parent)
 {
     struct d3d_sampler_state *state = parent;
-    struct d3d_device *device = impl_from_ID3D11Device(state->device);
+    struct d3d_device *device = impl_from_ID3D11Device2(state->device);
 
     wine_rb_remove(&device->sampler_states, &state->entry);
     wined3d_private_store_cleanup(&state->private_store);
-    HeapFree(GetProcessHeap(), 0, parent);
+    heap_free(parent);
 }
 
 static const struct wined3d_parent_ops d3d_sampler_wined3d_parent_ops =
@@ -1478,8 +1494,7 @@ static HRESULT d3d_sampler_state_init(struct d3d_sampler_state *state, struct d3
         return hr;
     }
 
-    state->device = &device->ID3D11Device_iface;
-    ID3D11Device_AddRef(state->device);
+    ID3D11Device2_AddRef(state->device = &device->ID3D11Device2_iface);
 
     return S_OK;
 }
@@ -1518,7 +1533,7 @@ HRESULT d3d_sampler_state_create(struct d3d_device *device, const D3D11_SAMPLER_
         return S_OK;
     }
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+    if (!(object = heap_alloc_zero(sizeof(*object))))
     {
         wined3d_mutex_unlock();
         return E_OUTOFMEMORY;
@@ -1529,7 +1544,7 @@ HRESULT d3d_sampler_state_create(struct d3d_device *device, const D3D11_SAMPLER_
     if (FAILED(hr))
     {
         WARN("Failed to initialize sampler state, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
+        heap_free(object);
         return hr;
     }
 
