@@ -385,9 +385,10 @@ static inline jsval_t steal_ret(call_frame_t *frame)
     return r;
 }
 
-static inline void clear_ret(call_frame_t *frame)
+static inline void clear_acc(script_ctx_t *ctx)
 {
-    jsval_release(steal_ret(frame));
+    jsval_release(ctx->acc);
+    ctx->acc = jsval_undefined();
 }
 
 static HRESULT scope_push(scope_chain_t *scope, jsdisp_t *jsobj, IDispatch *obj, scope_chain_t **ret)
@@ -520,7 +521,7 @@ static HRESULT disp_cmp(IDispatch *disp1, IDispatch *disp2, BOOL *ret)
 }
 
 /* ECMA-262 3rd Edition    11.9.6 */
-static HRESULT equal2_values(jsval_t lval, jsval_t rval, BOOL *ret)
+HRESULT jsval_strict_equal(jsval_t lval, jsval_t rval, BOOL *ret)
 {
     jsval_type_t type = jsval_type(lval);
 
@@ -849,7 +850,7 @@ static HRESULT interp_case(script_ctx_t *ctx)
     TRACE("\n");
 
     v = stack_pop(ctx);
-    hres = equal2_values(stack_top(ctx), v, &b);
+    hres = jsval_strict_equal(stack_top(ctx), v, &b);
     jsval_release(v);
     if(FAILED(hres))
         return hres;
@@ -1155,7 +1156,6 @@ static HRESULT interp_refval(script_ctx_t *ctx)
 static HRESULT interp_new(script_ctx_t *ctx)
 {
     const unsigned argc = get_op_uint(ctx, 0);
-    call_frame_t *frame = ctx->call_ctx;
     jsval_t constr;
 
     TRACE("%d\n", argc);
@@ -1171,9 +1171,9 @@ static HRESULT interp_new(script_ctx_t *ctx)
     else if(!get_object(constr))
         return throw_type_error(ctx, JS_E_INVALID_PROPERTY, NULL);
 
-    clear_ret(frame);
+    clear_acc(ctx);
     return disp_call_value(ctx, get_object(constr), NULL, DISPATCH_CONSTRUCT | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
-                           argc, stack_args(ctx, argc), &frame->ret);
+                           argc, stack_args(ctx, argc), &ctx->acc);
 }
 
 /* ECMA-262 3rd Edition    11.2.3 */
@@ -1181,7 +1181,6 @@ static HRESULT interp_call(script_ctx_t *ctx)
 {
     const unsigned argn = get_op_uint(ctx, 0);
     const int do_ret = get_op_int(ctx, 1);
-    call_frame_t *frame = ctx->call_ctx;
     jsval_t obj;
 
     TRACE("%d %d\n", argn, do_ret);
@@ -1190,9 +1189,9 @@ static HRESULT interp_call(script_ctx_t *ctx)
     if(!is_object_instance(obj))
         return throw_type_error(ctx, JS_E_INVALID_PROPERTY, NULL);
 
-    clear_ret(frame);
+    clear_acc(ctx);
     return disp_call_value(ctx, get_object(obj), NULL, DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
-                           argn, stack_args(ctx, argn), do_ret ? &frame->ret : NULL);
+                           argn, stack_args(ctx, argn), do_ret ? &ctx->acc : NULL);
 }
 
 /* ECMA-262 3rd Edition    11.2.3 */
@@ -1200,7 +1199,6 @@ static HRESULT interp_call_member(script_ctx_t *ctx)
 {
     const unsigned argn = get_op_uint(ctx, 0);
     const int do_ret = get_op_int(ctx, 1);
-    call_frame_t *frame = ctx->call_ctx;
     exprval_t ref;
 
     TRACE("%d %d\n", argn, do_ret);
@@ -1208,9 +1206,9 @@ static HRESULT interp_call_member(script_ctx_t *ctx)
     if(!stack_topn_exprval(ctx, argn, &ref))
         return throw_type_error(ctx, ref.u.hres, NULL);
 
-    clear_ret(frame);
+    clear_acc(ctx);
     return exprval_call(ctx, &ref, DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
-            argn, stack_args(ctx, argn), do_ret ? &frame->ret : NULL);
+            argn, stack_args(ctx, argn), do_ret ? &ctx->acc : NULL);
 }
 
 /* ECMA-262 3rd Edition    11.1.1 */
@@ -1296,7 +1294,7 @@ static HRESULT interp_local(script_ctx_t *ctx)
     jsval_t copy;
     HRESULT hres;
 
-    TRACE("%d\n", arg);
+    TRACE("%d: %s\n", arg, debugstr_w(local_name(frame, arg)));
 
     if(!frame->base_scope || !frame->base_scope->frame)
         return identifier_value(ctx, local_name(frame, arg));
@@ -1399,8 +1397,6 @@ static HRESULT interp_carray(script_ctx_t *ctx)
 {
     const unsigned arg = get_op_uint(ctx, 0);
     jsdisp_t *array;
-    jsval_t val;
-    unsigned i;
     HRESULT hres;
 
     TRACE("%u\n", arg);
@@ -1409,20 +1405,27 @@ static HRESULT interp_carray(script_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    i = arg;
-    while(i--) {
-        val = stack_pop(ctx);
-        hres = jsdisp_propput_idx(array, i, val);
-        jsval_release(val);
-        if(FAILED(hres)) {
-            jsdisp_release(array);
-            return hres;
-        }
-    }
-
     return stack_push(ctx, jsval_obj(array));
 }
 
+static HRESULT interp_carray_set(script_ctx_t *ctx)
+{
+    const unsigned index = get_op_uint(ctx, 0);
+    jsval_t value, array;
+    HRESULT hres;
+
+    value = stack_pop(ctx);
+
+    TRACE("[%u] = %s\n", index, debugstr_jsval(value));
+
+    array = stack_top(ctx);
+    assert(is_object_instance(array));
+
+    hres = jsdisp_propput_idx(iface_to_jsdisp(get_object(array)), index, value);
+    jsval_release(value);
+    return hres;
+}
+
 /* ECMA-262 3rd Edition    11.1.5 */
 static HRESULT interp_new_obj(script_ctx_t *ctx)
 {
@@ -2098,7 +2101,7 @@ static HRESULT interp_preinc(script_ctx_t *ctx)
 static HRESULT equal_values(script_ctx_t *ctx, jsval_t lval, jsval_t rval, BOOL *ret)
 {
     if(jsval_type(lval) == jsval_type(rval) || (is_number(lval) && is_number(rval)))
-       return equal2_values(lval, rval, ret);
+       return jsval_strict_equal(lval, rval, ret);
 
     /* FIXME: NULL disps should be handled in more general way */
     if(is_object_instance(lval) && !get_object(lval))
@@ -2228,7 +2231,7 @@ static HRESULT interp_eq2(script_ctx_t *ctx)
 
     TRACE("%s === %s\n", debugstr_jsval(l), debugstr_jsval(r));
 
-    hres = equal2_values(r, l, &b);
+    hres = jsval_strict_equal(r, l, &b);
     jsval_release(l);
     jsval_release(r);
     if(FAILED(hres))
@@ -2249,7 +2252,7 @@ static HRESULT interp_neq2(script_ctx_t *ctx)
     r = stack_pop(ctx);
     l = stack_pop(ctx);
 
-    hres = equal2_values(r, l, &b);
+    hres = jsval_strict_equal(r, l, &b);
     jsval_release(l);
     jsval_release(r);
     if(FAILED(hres))
@@ -2595,16 +2598,15 @@ static HRESULT interp_setret(script_ctx_t *ctx)
     return S_OK;
 }
 
-static HRESULT interp_push_ret(script_ctx_t *ctx)
+static HRESULT interp_push_acc(script_ctx_t *ctx)
 {
-    call_frame_t *frame = ctx->call_ctx;
     HRESULT hres;
 
     TRACE("\n");
 
-    hres = stack_push(ctx, frame->ret);
+    hres = stack_push(ctx, ctx->acc);
     if(SUCCEEDED(hres))
-        frame->ret = jsval_undefined();
+        ctx->acc = jsval_undefined();
     return hres;
 }
 
@@ -2791,8 +2793,8 @@ static HRESULT enter_bytecode(script_ctx_t *ctx, jsval_t *r)
             assert(frame->scope == frame->base_scope);
 
             if(return_to_interp) {
-                clear_ret(frame->prev_frame);
-                frame->prev_frame->ret = steal_ret(frame);
+                jsval_release(ctx->acc);
+                ctx->acc = steal_ret(frame);
             }else if(r) {
                 *r = steal_ret(frame);
             }
