@@ -37,9 +37,6 @@
 #include "wine/test.h"
 
 static HRESULT (WINAPI *pMFCreateSourceResolver)(IMFSourceResolver **resolver);
-static HRESULT (WINAPI *pMFCreateMFByteStreamOnStream)(IStream *stream, IMFByteStream **bytestream);
-static HRESULT (WINAPI *pMFCreateMemoryBuffer)(DWORD max_length, IMFMediaBuffer **buffer);
-
 
 DEFINE_GUID(MF_READWRITE_ENABLE_HARDWARE_TRANSFORMS, 0xa634a91c, 0x822b, 0x41b9, 0xa4, 0x94, 0x4d, 0xe4, 0x64, 0x36, 0x12, 0xb0);
 
@@ -197,8 +194,6 @@ static void init_functions(void)
 
 #define X(f) if (!(p##f = (void*)GetProcAddress(mod, #f))) return;
     X(MFCreateSourceResolver);
-    X(MFCreateMFByteStreamOnStream);
-    X(MFCreateMemoryBuffer);
 #undef X
 }
 
@@ -207,8 +202,11 @@ static void test_MFCreateMediaType(void)
     HRESULT hr;
     IMFMediaType *mediatype;
 
+    hr = MFStartup(MAKELONG( MF_API_VERSION, 0xdead ), MFSTARTUP_FULL);
+    ok(hr == MF_E_BAD_STARTUP_VERSION, "got 0x%08x\n", hr);
+
     hr = MFStartup(MF_VERSION, MFSTARTUP_FULL);
-    todo_wine ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
 
 if(0)
 {
@@ -252,129 +250,6 @@ static void test_MFCreateAttributes(void)
     IMFAttributes_Release(attributes);
 }
 
-static void test_MFCreateMFByteStreamOnStream(void)
-{
-    IMFByteStream *bytestream;
-    IStream *stream;
-    HRESULT hr;
-
-    if(!pMFCreateMFByteStreamOnStream)
-    {
-        win_skip("MFCreateMFByteStreamOnStream() not found\n");
-        return;
-    }
-
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = pMFCreateMFByteStreamOnStream(stream, &bytestream );
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    IStream_Release(stream);
-    IMFByteStream_Release(bytestream);
-}
-
-static void test_MFCreateMemoryBuffer(void)
-{
-    IMFMediaBuffer *buffer;
-    HRESULT hr;
-    DWORD length, max;
-    BYTE *data, *data2;
-
-    if(!pMFCreateMemoryBuffer)
-    {
-        win_skip("MFCreateMemoryBuffer() not found\n");
-        return;
-    }
-
-    hr = pMFCreateMemoryBuffer(1024, NULL);
-    ok(hr == E_INVALIDARG || hr == E_POINTER, "got 0x%08x\n", hr);
-
-    hr = pMFCreateMemoryBuffer(0, &buffer);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-    if(buffer)
-    {
-        hr = IMFMediaBuffer_GetMaxLength(buffer, &length);
-        ok(hr == S_OK, "got 0x%08x\n", hr);
-        ok(length == 0, "got %u\n", length);
-
-        IMFMediaBuffer_Release(buffer);
-    }
-
-    hr = pMFCreateMemoryBuffer(1024, &buffer);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_GetMaxLength(buffer, NULL);
-    ok(hr == E_INVALIDARG || hr == E_POINTER, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_GetMaxLength(buffer, &length);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-    ok(length == 1024, "got %u\n", length);
-
-    hr = IMFMediaBuffer_SetCurrentLength(buffer, 1025);
-    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_SetCurrentLength(buffer, 10);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_GetCurrentLength(buffer, NULL);
-    ok(hr == E_INVALIDARG || hr == E_POINTER, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_GetCurrentLength(buffer, &length);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-    ok(length == 10, "got %u\n", length);
-
-    length = 0;
-    max = 0;
-    hr = IMFMediaBuffer_Lock(buffer, NULL, &length, &max);
-    ok(hr == E_INVALIDARG || hr == E_POINTER, "got 0x%08x\n", hr);
-    ok(length == 0, "got %u\n", length);
-    ok(max == 0, "got %u\n", length);
-
-    hr = IMFMediaBuffer_Lock(buffer, &data, &max, &length);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-    ok(length == 10, "got %u\n", length);
-    ok(max == 1024, "got %u\n", max);
-
-    /* Attempt to lock the bufer twice */
-    hr = IMFMediaBuffer_Lock(buffer, &data2, &max, &length);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-    ok(data == data2, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_Unlock(buffer);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_Unlock(buffer);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_Lock(buffer, &data, NULL, NULL);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IMFMediaBuffer_Unlock(buffer);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    /* Extra Unlock */
-    hr = IMFMediaBuffer_Unlock(buffer);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    IMFMediaBuffer_Release(buffer);
-}
-
-static void test_MFSample(void)
-{
-    IMFSample *sample;
-    HRESULT hr;
-    UINT32 count;
-
-    hr = MFCreateSample( &sample );
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IMFSample_GetBufferCount(sample, &count);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-    ok(count == 0, "got %d\n", count);
-
-    IMFSample_Release(sample);
-}
 
 START_TEST(mfplat)
 {
@@ -386,9 +261,6 @@ START_TEST(mfplat)
     test_source_resolver();
     test_MFCreateMediaType();
     test_MFCreateAttributes();
-    test_MFSample();
-    test_MFCreateMFByteStreamOnStream();
-    test_MFCreateMemoryBuffer();
 
     CoUninitialize();
 }
