@@ -359,6 +359,14 @@ HANDLE WINAPI WinSqmStartSession(GUID *sessionguid, DWORD sessionid, DWORD unkno
     return INVALID_HANDLE_VALUE;
 }
 
+/***********************************************************************
+ *          WinSqmSetDWORD (NTDLL.@)
+ */
+void WINAPI WinSqmSetDWORD(HANDLE session, DWORD datapoint_id, DWORD datapoint_value)
+{
+    FIXME("(%p, %d, %d): stub\n", session, datapoint_id, datapoint_value);
+}
+
 /******************************************************************************
  *                  EtwEventRegister (NTDLL.@)
  */
@@ -476,3 +484,11 @@ ULONG WINAPI EtwEventWrite( REGHANDLE handle, const EVENT_DESCRIPTOR *descriptor
     FIXME("(%s, %p, %u, %p): stub\n", wine_dbgstr_longlong(handle), descriptor, count, data);
     return ERROR_SUCCESS;
 }
+
+/***********************************************************************
+ *		    DbgUiRemoteBreakin (NTDLL.@)
+ */
+void WINAPI DbgUiRemoteBreakin( void *arg )
+{
+    FIXME("stub\n");
+}
