@@ -26,6 +26,8 @@
 #include "shlobj.h"
 #include "shlwapi.h"
 #include "winsvc.h"
+
+#include "wine/heap.h"
 #include "wine/test.h"
 
 #include "initguid.h"
@@ -159,7 +161,7 @@ static void test_namespace(void)
     ok(folder == NULL, "expected NULL, got %p\n", folder);
 
     /* test valid folder ids */
-    for (i = 0; i < sizeof(special_folders)/sizeof(special_folders[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(special_folders); i++)
     {
         V_VT(&var) = VT_I4;
         V_I4(&var) = special_folders[i];
@@ -276,7 +278,7 @@ static void test_namespace(void)
     GetFullPathNameW(winetestW, MAX_PATH, tempW, NULL);
 
     len = GetLongPathNameW(tempW, NULL, 0);
-    long_pathW = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+    long_pathW = heap_alloc(len * sizeof(WCHAR));
     GetLongPathNameW(tempW, long_pathW, len);
 
     V_VT(&var) = VT_BSTR;
@@ -348,7 +350,7 @@ static void test_namespace(void)
         SysFreeString(V_BSTR(&var));
     }
 
-    HeapFree(GetProcessHeap(), 0, long_pathW);
+    heap_free(long_pathW);
     RemoveDirectoryW(winetestW);
     SetCurrentDirectoryW(curW);
     IShellDispatch_Release(sd);
@@ -472,7 +474,7 @@ static void test_items(void)
     ok(!item, "item is not null\n");
 
     /* create test files */
-    for (i = 0; i < sizeof(file_defs)/sizeof(file_defs[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(file_defs); i++)
     {
         switch (file_defs[i].type)
         {
@@ -539,8 +541,7 @@ static void test_items(void)
     count = -1;
     r = FolderItems_get_Count(items, &count);
     ok(r == S_OK, "FolderItems::get_Count failed: %08x\n", r);
-    ok(count == sizeof(file_defs)/sizeof(file_defs[0]),
-       "expected %d files, got %d\n", (LONG)(sizeof(file_defs)/sizeof(file_defs[0])), count);
+    ok(count == ARRAY_SIZE(file_defs), "got %d files\n", count);
 
     V_VT(&var) = VT_EMPTY;
     item = (FolderItem*)0xdeadbeef;
@@ -604,7 +605,7 @@ static void test_items(void)
     V_VT(&int_index) = VT_I4;
 
     /* test the folder item corresponding to each file */
-    for (i = 0; i < sizeof(file_defs)/sizeof(file_defs[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(file_defs); i++)
     {
         VARIANT_BOOL b;
         BSTR name;
@@ -728,7 +729,7 @@ static void test_items(void)
     }
 
     /* test that there are only as many folder items as there were files */
-    V_I4(&int_index) = sizeof(file_defs)/sizeof(file_defs[0]);
+    V_I4(&int_index) = ARRAY_SIZE(file_defs);
     item = (FolderItem*)0xdeadbeef;
     r = FolderItems_Item(items, int_index, &item);
     ok(r == S_FALSE, "expected S_FALSE, got %08x\n", r);
@@ -974,7 +975,7 @@ if (0) /* crashes on pre-vista */ {
     IShellView_Release(view);
 
     /* Try with some other folder, that's not a desktop */
-    GetTempPathW(sizeof(pathW)/sizeof(pathW[0]), pathW);
+    GetTempPathW(ARRAY_SIZE(pathW), pathW);
     hr = IShellFolder_ParseDisplayName(desktop, NULL, NULL, pathW, NULL, &pidl, NULL);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
@@ -1217,7 +1218,7 @@ static void test_ParseName(void)
         &IID_IShellDispatch, (void**)&sd);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    GetTempPathW(sizeof(pathW)/sizeof(pathW[0]), pathW);
+    GetTempPathW(ARRAY_SIZE(pathW), pathW);
     V_VT(&v) = VT_BSTR;
     V_BSTR(&v) = SysAllocString(pathW);
     hr = IShellDispatch_NameSpace(sd, v, &folder);
@@ -1286,7 +1287,7 @@ static void test_Verbs(void)
         &IID_IShellDispatch, (void**)&sd);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    GetTempPathW(sizeof(pathW)/sizeof(pathW[0]), pathW);
+    GetTempPathW(ARRAY_SIZE(pathW), pathW);
     V_VT(&v) = VT_BSTR;
     V_BSTR(&v) = SysAllocString(pathW);
     hr = IShellDispatch_NameSpace(sd, v, &folder);
