@@ -918,6 +918,7 @@ typedef struct WINE_MIDIStream {
     DWORD			dwPositionMS;
     DWORD			dwPulses;
     DWORD			dwStartTicks;
+    DWORD			dwElapsedMS;
     WORD			wFlags;
     WORD			status;
     HANDLE			hEvent;
@@ -973,6 +974,20 @@ static	DWORD	MMSYSTEM_MidiStream_Convert(WINE_MIDIStream* lpMidiStrm, DWORD puls
     return ret;
 }
 
+static DWORD midistream_get_playing_position(WINE_MIDIStream* lpMidiStrm)
+{
+    switch (lpMidiStrm->status) {
+    case MSM_STATUS_STOPPED:
+    case MSM_STATUS_PAUSED:
+        return lpMidiStrm->dwElapsedMS;
+    case MSM_STATUS_PLAYING:
+        return GetTickCount() - lpMidiStrm->dwStartTicks;
+    default:
+        FIXME("Unknown playing status %hu\n", lpMidiStrm->status);
+        return 0;
+    }
+}
+
 /**************************************************************************
  * 			MMSYSTEM_MidiStream_MessageHandler	[internal]
  */
@@ -988,7 +1003,11 @@ static	BOOL	MMSYSTEM_MidiStream_MessageHandler(WINE_MIDIStream* lpMidiStrm, LPWI
             return FALSE;
         case WINE_MSM_STOP:
             TRACE("STOP\n");
+            EnterCriticalSection(&lpMidiStrm->lock);
             lpMidiStrm->status = MSM_STATUS_STOPPED;
+            lpMidiStrm->dwPulses = 0;
+            lpMidiStrm->dwElapsedMS = 0;
+            LeaveCriticalSection(&lpMidiStrm->lock);
             /* this is not quite what MS doc says... */
             midiOutReset(lpMidiStrm->hDevice);
             /* empty list of already submitted buffers */
@@ -1004,15 +1023,27 @@ static	BOOL	MMSYSTEM_MidiStream_MessageHandler(WINE_MIDIStream* lpMidiStrm, LPWI
                                (HDRVR)lpMidiStrm->hDevice, MM_MOM_DONE,
                                lpwm->mod.dwInstance, (DWORD_PTR)lphdr, 0);
             }
+            SetEvent((HANDLE)msg->wParam);
             return TRUE;
         case WINE_MSM_RESUME:
             /* FIXME: send out cc64 0 (turn off sustain pedal) on every channel */
-            lpMidiStrm->dwStartTicks = GetTickCount() - lpMidiStrm->dwPositionMS;
-            lpMidiStrm->status = MSM_STATUS_PLAYING;
+            if (lpMidiStrm->status != MSM_STATUS_PLAYING) {
+                EnterCriticalSection(&lpMidiStrm->lock);
+                lpMidiStrm->dwStartTicks = GetTickCount() - lpMidiStrm->dwElapsedMS;
+                lpMidiStrm->status = MSM_STATUS_PLAYING;
+                LeaveCriticalSection(&lpMidiStrm->lock);
+            }
+            SetEvent((HANDLE)msg->wParam);
             return TRUE;
         case WINE_MSM_PAUSE:
             /* FIXME: send out cc64 0 (turn off sustain pedal) on every channel */
-            lpMidiStrm->status = MSM_STATUS_PAUSED;
+            if (lpMidiStrm->status != MSM_STATUS_PAUSED) {
+                EnterCriticalSection(&lpMidiStrm->lock);
+                lpMidiStrm->dwElapsedMS = GetTickCount() - lpMidiStrm->dwStartTicks;
+                lpMidiStrm->status = MSM_STATUS_PAUSED;
+                LeaveCriticalSection(&lpMidiStrm->lock);
+            }
+            SetEvent((HANDLE)msg->wParam);
             break;
 	/* FIXME(EPP): "I don't understand the content of the first MIDIHDR sent
 	 * by native mcimidi, it doesn't look like a correct one".
@@ -1166,6 +1197,8 @@ start_header:
 			    goto start_header;
 			}
 		    }
+		    /* reset dwToGo because dwStartTicks might be updated */
+		    dwToGo = lpMidiStrm->dwStartTicks + lpMidiStrm->dwPositionMS;
 		} else {
 		    /* timeout, so me->dwDeltaTime is elapsed, can break the while loop */
 		    break;
@@ -1299,6 +1332,7 @@ MMRESULT WINAPI midiStreamOpen(HMIDISTRM* lphMidiStrm, LPUINT lpuDeviceID,
     lpMidiStrm->dwTimeDiv = 24;    /* ticks per quarter note */
     lpMidiStrm->dwPositionMS = 0;
     lpMidiStrm->status = MSM_STATUS_PAUSED;
+    lpMidiStrm->dwElapsedMS = 0;
 
     mosm.dwStreamID = (DWORD)lpMidiStrm;
     /* FIXME: the correct value is not allocated yet for MAPPER */
@@ -1387,20 +1421,49 @@ MMRESULT WINAPI midiStreamOut(HMIDISTRM hMidiStrm, LPMIDIHDR lpMidiHdr,
     return ret;
 }
 
+static MMRESULT midistream_post_message_and_wait(WINE_MIDIStream* lpMidiStrm, UINT msg, LPARAM lParam)
+{
+    HANDLE hObjects[2];
+
+    hObjects[0] = CreateEventW(NULL, FALSE, FALSE, NULL);
+    if (!hObjects[0])
+        return MMSYSERR_ERROR;
+
+    if (!PostThreadMessageA(lpMidiStrm->dwThreadID, msg, (WPARAM)hObjects[0], lParam)) {
+        WARN("bad PostThreadMessage\n");
+        CloseHandle(hObjects[0]);
+        return MMSYSERR_ERROR;
+    }
+
+    if (GetCurrentThreadId() != lpMidiStrm->dwThreadID) {
+        DWORD ret;
+        hObjects[1] = lpMidiStrm->hThread;
+        ret = WaitForMultipleObjects(sizeof(hObjects)/sizeof(hObjects[0]), hObjects,
+                                     FALSE, INFINITE);
+        if (ret != WAIT_OBJECT_0) {
+            CloseHandle(hObjects[0]);
+            WARN("bad WaitForSingleObject (%u)\n", ret);
+            return MMSYSERR_ERROR;
+        }
+    }
+
+    CloseHandle(hObjects[0]);
+
+    return MMSYSERR_NOERROR;
+}
+
 /**************************************************************************
  * 				midiStreamPause			[WINMM.@]
  */
 MMRESULT WINAPI midiStreamPause(HMIDISTRM hMidiStrm)
 {
     WINE_MIDIStream*	lpMidiStrm;
-    DWORD		ret = MMSYSERR_NOERROR;
 
     TRACE("(%p)!\n", hMidiStrm);
 
     if (!MMSYSTEM_GetMidiStream(hMidiStrm, &lpMidiStrm, NULL))
         return MMSYSERR_INVALHANDLE;
-    PostThreadMessageA(lpMidiStrm->dwThreadID, WINE_MSM_PAUSE, 0, 0);
-    return ret;
+    return midistream_post_message_and_wait(lpMidiStrm, WINE_MSM_PAUSE, 0);
 }
 
 /**************************************************************************
@@ -1428,7 +1491,7 @@ MMRESULT WINAPI midiStreamPosition(HMIDISTRM hMidiStrm, LPMMTIME lpMMT, UINT cbm
 	    lpMMT->wType = TIME_MS;
 	    /* fall through to alternative format */
 	case TIME_MS:
-	    lpMMT->u.ms = lpMidiStrm->dwPositionMS;
+	    lpMMT->u.ms = midistream_get_playing_position(lpMidiStrm);
 	    TRACE("=> %d ms\n", lpMMT->u.ms);
 	    break;
 	case TIME_TICKS:
@@ -1496,14 +1559,12 @@ MMRESULT WINAPI midiStreamProperty(HMIDISTRM hMidiStrm, LPBYTE lpPropData, DWORD
 MMRESULT WINAPI midiStreamRestart(HMIDISTRM hMidiStrm)
 {
     WINE_MIDIStream*	lpMidiStrm;
-    MMRESULT		ret = MMSYSERR_NOERROR;
 
     TRACE("(%p)!\n", hMidiStrm);
 
     if (!MMSYSTEM_GetMidiStream(hMidiStrm, &lpMidiStrm, NULL))
         return MMSYSERR_INVALHANDLE;
-    PostThreadMessageA(lpMidiStrm->dwThreadID, WINE_MSM_RESUME, 0, 0);
-    return ret;
+    return midistream_post_message_and_wait(lpMidiStrm, WINE_MSM_RESUME, 0);
 }
 
 /**************************************************************************
@@ -1512,14 +1573,12 @@ MMRESULT WINAPI midiStreamRestart(HMIDISTRM hMidiStrm)
 MMRESULT WINAPI midiStreamStop(HMIDISTRM hMidiStrm)
 {
     WINE_MIDIStream*	lpMidiStrm;
-    MMRESULT		ret = MMSYSERR_NOERROR;
 
     TRACE("(%p)!\n", hMidiStrm);
 
     if (!MMSYSTEM_GetMidiStream(hMidiStrm, &lpMidiStrm, NULL))
         return MMSYSERR_INVALHANDLE;
-    PostThreadMessageA(lpMidiStrm->dwThreadID, WINE_MSM_STOP, 0, 0);
-    return ret;
+    return midistream_post_message_and_wait(lpMidiStrm, WINE_MSM_STOP, 0);
 }
 
 struct mm_starter
