@@ -47,10 +47,33 @@ static const char testProbeMessage[] = "<?xml version=\"1.0\" encoding=\"utf-8\"
     "xmlns:wsd=\"http://schemas.xmlsoap.org/ws/2005/04/discovery\" "
     "xmlns:grog=\"http://more.tests/\"><soap:Header><wsa:To>urn:schemas-xmlsoap-org:ws:2005:04:discovery</wsa:To>"
     "<wsa:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/Probe</wsa:Action>"
-    "<wsa:MessageID>urn:uuid:aa3eb169-9440-4eb7-9090-88402664cc6a</wsa:MessageID></soap:Header>"
-    "<soap:Body><wsd:Probe><wsd:Types>grog:Cider</wsd:Types></wsd:Probe></soap:Body></soap:Envelope>";
+    "<wsa:MessageID>urn:uuid:%s</wsa:MessageID>"
+    "<wsd:AppSequence InstanceId=\"21\" SequenceId=\"urn:uuid:638abee8-124d-4b6a-8b85-8cf2837a2fd2\" MessageNumber=\"14\"></wsd:AppSequence>"
+    "<grog:Perry>ExtraInfo</grog:Perry></soap:Header>"
+    "<soap:Body><wsd:Probe><wsd:Types>grog:Cider</wsd:Types><grog:Lager>MoreInfo</grog:Lager></wsd:Probe></soap:Body></soap:Envelope>";
+
+static const WCHAR discoveryTo[] = {
+    'u','r','n',':',
+    's','c','h','e','m','a','s','-','x','m','l','s','o','a','p','-','o','r','g',':',
+    'w','s',':','2','0','0','5',':','0','4',':',
+    'd','i','s','c','o','v','e','r','y', 0 };
+
+static const WCHAR actionProbe[] = {
+    'h','t','t','p',':','/','/',
+    's','c','h','e','m','a','s','.','x','m','l','s','o','a','p','.','o','r','g','/',
+    'w','s','/','2','0','0','5','/','0','4','/',
+    'd','i','s','c','o','v','e','r','y','/',
+    'P','r','o','b','e', 0 };
+
+static const WCHAR uri_more_tests[] = { 'h','t','t','p',':','/','/','m','o','r','e','.','t','e','s','t','s','/', 0 };
+static const WCHAR uri_more_tests_no_slash[] = { 'h','t','t','p',':','/','/','m','o','r','e','.','t','e','s','t','s', 0 };
+static const WCHAR prefix_grog[] = { 'g','r','o','g', 0 };
+static const WCHAR name_cider[] = { 'C','i','d','e','r', 0 };
 
 static HANDLE probe_event = NULL;
+static UUID probe_message_id;
+
+static IWSDiscoveryPublisher *publisher_instance = NULL;
 
 #define MAX_CACHED_MESSAGES     5
 #define MAX_LISTENING_THREADS  20
@@ -211,6 +234,49 @@ static DWORD WINAPI listening_thread(LPVOID lpParam)
     return 0;
 }
 
+static BOOL start_listening_udp_unicast(messageStorage *msgStorage, struct sockaddr_in *address)
+{
+    listenerThreadParams *parameter = NULL;
+    const DWORD receive_timeout = 500;
+    const UINT reuse_addr = 1;
+    HANDLE hThread;
+    SOCKET s = 0;
+
+    /* Create the socket */
+    s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+    if (s == INVALID_SOCKET) goto cleanup;
+
+    /* Ensure the socket can be reused */
+    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (const char *)&reuse_addr, sizeof(reuse_addr)) == SOCKET_ERROR) goto cleanup;
+
+    /* Bind the socket to the local interface so we can receive data */
+    if (bind(s, (struct sockaddr *) address, sizeof(struct sockaddr_in)) == SOCKET_ERROR) goto cleanup;
+
+    /* Set a 500ms receive timeout */
+    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, (const char *) &receive_timeout, sizeof(receive_timeout)) == SOCKET_ERROR)
+        goto cleanup;
+
+    /* Allocate memory for thread parameters */
+    parameter = heap_alloc(sizeof(listenerThreadParams));
+
+    parameter->msgStorage = msgStorage;
+    parameter->listeningSocket = s;
+
+    hThread = CreateThread(NULL, 0, listening_thread, parameter, 0, NULL);
+    if (hThread == NULL) goto cleanup;
+
+    msgStorage->threadHandles[msgStorage->numThreadHandles] = hThread;
+    msgStorage->numThreadHandles++;
+
+    return TRUE;
+
+cleanup:
+    closesocket(s);
+    heap_free(parameter);
+
+    return FALSE;
+}
+
 static void start_listening(messageStorage *msgStorage, const char *multicastAddress, const char *bindAddress)
 {
     struct addrinfo *multicastAddr = NULL, *bindAddr = NULL, *interfaceAddr = NULL;
@@ -448,6 +514,53 @@ static ULONG WINAPI IWSDiscoveryPublisherNotifyImpl_Release(IWSDiscoveryPublishe
     return ref;
 }
 
+static void verify_wsdxml_name(const char *debug_prefix, WSDXML_NAME *name, LPCWSTR uri, LPCWSTR prefix,
+    LPCWSTR local_name)
+{
+    ok(name != NULL, "%s: name == NULL\n", debug_prefix);
+    if (name == NULL) return;
+
+    ok(name->LocalName != NULL && lstrcmpW(name->LocalName, local_name) == 0,
+        "%s: Local name = '%s'\n", debug_prefix, wine_dbgstr_w(name->LocalName));
+
+    ok(name->Space != NULL, "%s: Space == NULL\n", debug_prefix);
+    if (name->Space == NULL) return;
+
+    ok(name->Space->Uri != NULL && lstrcmpW(name->Space->Uri, uri) == 0,
+        "%s: URI == '%s'\n", debug_prefix, wine_dbgstr_w(name->Space->Uri));
+    ok(name->Space->PreferredPrefix != NULL && lstrcmpW(name->Space->PreferredPrefix, prefix) == 0,
+        "%s: Prefix = '%s'\n", debug_prefix, wine_dbgstr_w(name->Space->PreferredPrefix));
+}
+
+static void verify_wsdxml_any_text(const char *debug_prefix, WSDXML_ELEMENT *any, LPCWSTR uri, LPCWSTR prefix,
+    LPCWSTR local_name, LPCWSTR value)
+{
+    WSDXML_TEXT *child;
+
+    ok(any != NULL, "%s: any == NULL\n", debug_prefix);
+    if (any == NULL) return;
+
+    child = (WSDXML_TEXT *) any->FirstChild;
+
+    ok(any->Node.Type == ElementType, "%s: Node type == %d\n", debug_prefix, any->Node.Type);
+    ok(any->Node.Parent == NULL, "%s: Parent == %p\n", debug_prefix, any->Node.Parent);
+    ok(any->Node.Next == NULL, "%s: Next == %p\n", debug_prefix, any->Node.Next);
+    verify_wsdxml_name(debug_prefix, any->Name, uri, prefix, local_name);
+
+    ok(child != NULL, "%s: First child == NULL\n", debug_prefix);
+
+    if (child != NULL)
+    {
+        ok(child->Node.Type == TextType, "%s: Node type == %d\n", debug_prefix, child->Node.Type);
+        ok(child->Node.Parent == any, "%s: Parent == %p\n", debug_prefix, child->Node.Parent);
+        ok(child->Node.Next == NULL, "%s: Next == %p\n", debug_prefix, child->Node.Next);
+
+        if (child->Node.Type == TextType)
+            ok(child->Text != NULL && lstrcmpW(child->Text, value) == 0,
+                "%s: Text == '%s'\n", debug_prefix, wine_dbgstr_w(child->Text));
+    }
+}
+
 static HRESULT WINAPI IWSDiscoveryPublisherNotifyImpl_ProbeHandler(IWSDiscoveryPublisherNotify *This, const WSD_SOAP_MESSAGE *pSoap, IWSDMessageParameters *pMessageParameters)
 {
     trace("IWSDiscoveryPublisherNotifyImpl_ProbeHandler called (%p, %p, %p)\n", This, pSoap, pMessageParameters);
@@ -463,8 +576,214 @@ static HRESULT WINAPI IWSDiscoveryPublisherNotifyImpl_ProbeHandler(IWSDiscoveryP
 
     if (pSoap != NULL)
     {
+        static const WCHAR perry[] = {'P','e','r','r','y',0};
+        static const WCHAR extra_info[] = {'E','x','t','r','a','I','n','f','o',0};
+        WSD_PROBE *probe_msg = (WSD_PROBE *) pSoap->Body;
+        WSD_APP_SEQUENCE *appseq = (WSD_APP_SEQUENCE *) pSoap->Header.AppSequence;
+
         ok(pSoap->Body != NULL, "pSoap->Body == NULL\n");
-        ok(pSoap->Header.To != NULL, "pSoap->Header.To == NULL\n");
+        ok(pSoap->Header.To != NULL && lstrcmpW(pSoap->Header.To, discoveryTo) == 0,
+            "pSoap->Header.To == '%s'\n", wine_dbgstr_w(pSoap->Header.To));
+        ok(pSoap->Header.Action != NULL && lstrcmpW(pSoap->Header.Action, actionProbe) == 0,
+            "pSoap->Header.Action == '%s'\n", wine_dbgstr_w(pSoap->Header.Action));
+
+        ok(pSoap->Header.MessageID != NULL, "pSoap->Header.MessageID == NULL\n");
+
+        /* Ensure the message ID is at least 9 characters long (to skip past the 'urn:uuid:' prefix) */
+        if ((pSoap->Header.MessageID != NULL) && (lstrlenW(pSoap->Header.MessageID) > 9))
+        {
+            UUID uuid;
+            RPC_STATUS ret = UuidFromStringW((LPWSTR)pSoap->Header.MessageID + 9, &uuid);
+
+            trace("Received message with UUID '%s' (expected UUID '%s')\n", wine_dbgstr_guid(&uuid),
+                wine_dbgstr_guid(&probe_message_id));
+
+            /* Check if we've either received a message without a UUID, or the UUID isn't the one we sent. If so,
+               ignore it and wait for another message. */
+            if ((ret != RPC_S_OK) || (UuidEqual(&uuid, &probe_message_id, &ret) == FALSE)) return S_OK;
+        }
+
+        ok(appseq != NULL, "pSoap->Header.AppSequence == NULL\n");
+
+        if (appseq != NULL)
+        {
+            static const WCHAR seq_id[] = {'u','r','n',':','u','u','i','d',':','6','3','8','a','b','e','e','8','-',
+                '1','2','4','d','-','4','b','6','a','-','8','b','8','5','-',
+                '8','c','f','2','8','3','7','a','2','f','d','2',0};
+
+            ok(appseq->InstanceId == 21, "pSoap->Header.AppSequence->InstanceId = %s\n",
+                wine_dbgstr_longlong(appseq->InstanceId));
+            ok(lstrcmpW(appseq->SequenceId, seq_id) == 0, "pSoap->Header.AppSequence->SequenceId = '%s'\n",
+                wine_dbgstr_w(appseq->SequenceId));
+            ok(appseq->MessageNumber == 14, "pSoap->Header.AppSequence->MessageNumber = %s\n",
+                wine_dbgstr_longlong(appseq->MessageNumber));
+        }
+
+        verify_wsdxml_any_text("pSoap->Header.AnyHeaders", pSoap->Header.AnyHeaders, uri_more_tests_no_slash,
+            prefix_grog, perry, extra_info);
+
+        if (probe_msg != NULL)
+        {
+            static const WCHAR lager[] = {'L','a','g','e','r',0};
+            static const WCHAR more_info[] = {'M','o','r','e','I','n','f','o',0};
+            IWSDUdpAddress *remote_addr = NULL;
+            HRESULT rc;
+
+            ok(probe_msg->Types != NULL, "Probe message Types == NULL\n");
+
+            if (probe_msg->Types != NULL)
+            {
+                verify_wsdxml_name("probe_msg->Types->Element", probe_msg->Types->Element, uri_more_tests_no_slash,
+                    prefix_grog, name_cider);
+                ok(probe_msg->Types->Next == NULL, "probe_msg->Types->Next == %p\n", probe_msg->Types->Next);
+            }
+
+            ok(probe_msg->Scopes == NULL, "Probe message Scopes != NULL\n");
+            verify_wsdxml_any_text("probe_msg->Any", probe_msg->Any, uri_more_tests_no_slash, prefix_grog, lager, more_info);
+
+            rc = IWSDMessageParameters_GetRemoteAddress(pMessageParameters, (IWSDAddress **) &remote_addr);
+            ok(rc == S_OK, "IWSDMessageParameters_GetRemoteAddress returned %08x\n", rc);
+
+            if (remote_addr != NULL)
+            {
+                messageStorage *msg_storage;
+                char endpoint_reference_string[MAX_PATH], app_sequence_string[MAX_PATH];
+                LPWSTR publisherIdW = NULL, sequenceIdW = NULL;
+                SOCKADDR_STORAGE remote_sock;
+                WSDXML_ELEMENT *header_any_element, *body_any_element, *endpoint_any_element, *ref_param_any_element;
+                WSDXML_NAME header_any_name;
+                WSDXML_NAMESPACE ns;
+                static const WCHAR header_any_name_text[] = {'B','e','e','r',0};
+                static const WCHAR header_any_text[] = {'P','u','b','l','i','s','h','T','e','s','t',0};
+                static const WCHAR body_any_text[] = {'B','o','d','y','T','e','s','t',0};
+                static const WCHAR endpoint_any_text[] = {'E','n','d','P','T','e','s','t',0};
+                static const WCHAR ref_param_any_text[] = {'R','e','f','P','T','e','s','t',0};
+                static const WCHAR uri[] = {'h','t','t','p',':','/','/','w','i','n','e','.','t','e','s','t','/',0};
+                static const WCHAR prefix[] = {'w','i','n','e',0};
+                BOOL probe_matches_message_seen = FALSE, endpoint_reference_seen = FALSE, app_sequence_seen = FALSE;
+                BOOL metadata_version_seen = FALSE, wine_ns_seen = FALSE, body_probe_matches_seen = FALSE;
+                BOOL types_seen = FALSE, any_header_seen = FALSE, any_body_seen = FALSE;
+                BOOL message_ok;
+                char *msg;
+                int i;
+
+                rc = IWSDUdpAddress_GetSockaddr(remote_addr, &remote_sock);
+                ok(rc == S_OK, "IWSDMessageParameters_GetRemoteAddress returned %08x\n", rc);
+
+                IWSDUdpAddress_Release(remote_addr);
+
+                msg_storage = heap_alloc_zero(sizeof(messageStorage));
+                if (msg_storage == NULL) goto after_matchprobe_test;
+
+                msg_storage->running = TRUE;
+                InitializeCriticalSection(&msg_storage->criticalSection);
+
+                ok(start_listening_udp_unicast(msg_storage, (struct sockaddr_in *) &remote_sock) == TRUE,
+                    "Unable to listen on local socket for UDP messages\n");
+
+                publisherIdW = utf8_to_wide(publisherId);
+                sequenceIdW = utf8_to_wide(sequenceId);
+
+                /* Create "any" elements for header */
+                ns.Uri = uri;
+                ns.PreferredPrefix = prefix;
+
+                header_any_name.LocalName = (WCHAR *) header_any_name_text;
+                header_any_name.Space = &ns;
+
+                rc = WSDXMLBuildAnyForSingleElement(&header_any_name, header_any_text, &header_any_element);
+                ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+                rc = WSDXMLBuildAnyForSingleElement(&header_any_name, body_any_text, &body_any_element);
+                ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+                rc = WSDXMLBuildAnyForSingleElement(&header_any_name, endpoint_any_text, &endpoint_any_element);
+                ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+                rc = WSDXMLBuildAnyForSingleElement(&header_any_name, ref_param_any_text, &ref_param_any_element);
+                ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
+
+                rc = IWSDiscoveryPublisher_MatchProbeEx(publisher_instance, pSoap, pMessageParameters, publisherIdW, 1, 1, 1,
+                    sequenceIdW, probe_msg->Types, NULL, NULL, header_any_element, ref_param_any_element, NULL,
+                    endpoint_any_element, body_any_element);
+                todo_wine ok(rc == S_OK, "IWSDiscoveryPublisher_MatchProbe failed with %08x\n", rc);
+
+                WSDFreeLinkedMemory(header_any_element);
+                WSDFreeLinkedMemory(body_any_element);
+                WSDFreeLinkedMemory(endpoint_any_element);
+                WSDFreeLinkedMemory(ref_param_any_element);
+
+                /* Wait up to 2 seconds for messages to be received */
+                if (WaitForMultipleObjects(msg_storage->numThreadHandles, msg_storage->threadHandles, TRUE, 5000) == WAIT_TIMEOUT)
+                {
+                    /* Wait up to 1 more second for threads to terminate */
+                    msg_storage->running = FALSE;
+                    WaitForMultipleObjects(msg_storage->numThreadHandles, msg_storage->threadHandles, TRUE, 1000);
+                }
+
+                for (i = 0; i < msg_storage->numThreadHandles; i++)
+                {
+                    CloseHandle(msg_storage->threadHandles[i]);
+                }
+
+                DeleteCriticalSection(&msg_storage->criticalSection);
+
+                /* Verify we've received a message */
+                todo_wine ok(msg_storage->messageCount >= 1, "No messages received\n");
+
+                sprintf(endpoint_reference_string, "<wsa:EndpointReference><wsa:Address>%s</wsa:Address>"
+                    "<wsa:ReferenceParameters><wine:Beer>RefPTest</wine:Beer></wsa:ReferenceParameters>"
+                    "<wine:Beer>EndPTest</wine:Beer></wsa:EndpointReference>", publisherId);
+
+                sprintf(app_sequence_string, "<wsd:AppSequence InstanceId=\"1\" SequenceId=\"%s\" MessageNumber=\"1\"></wsd:AppSequence>",
+                    sequenceId);
+
+                message_ok = FALSE;
+
+                /* Check we're received the correct message */
+                for (i = 0; i < msg_storage->messageCount; i++)
+                {
+                    msg = msg_storage->messages[i];
+                    message_ok = FALSE;
+
+                    probe_matches_message_seen = (strstr(msg,
+                        "<wsa:Action>http://schemas.xmlsoap.org/ws/2005/04/discovery/ProbeMatches</wsa:Action>") != NULL);
+                    endpoint_reference_seen = (strstr(msg, endpoint_reference_string) != NULL);
+                    app_sequence_seen = (strstr(msg, app_sequence_string) != NULL);
+                    metadata_version_seen = (strstr(msg, "<wsd:MetadataVersion>1</wsd:MetadataVersion>") != NULL);
+                    any_header_seen = (strstr(msg, "<wine:Beer>PublishTest</wine:Beer>") != NULL);
+                    wine_ns_seen = (strstr(msg, "xmlns:grog=\"http://more.tests\"") != NULL);
+                    body_probe_matches_seen = (strstr(msg, "<soap:Body><wsd:ProbeMatches") != NULL);
+                    any_body_seen = (strstr(msg, "<wine:Beer>BodyTest</wine:Beer>") != NULL);
+                    types_seen = (strstr(msg, "<wsd:Types>grog:Cider</wsd:Types>") != NULL);
+                    message_ok = probe_matches_message_seen && endpoint_reference_seen && app_sequence_seen &&
+                        metadata_version_seen && body_probe_matches_seen && types_seen;
+
+                    if (message_ok) break;
+                }
+
+                for (i = 0; i < msg_storage->messageCount; i++)
+                {
+                    heap_free(msg_storage->messages[i]);
+                }
+
+                todo_wine ok(probe_matches_message_seen == TRUE, "Probe matches message not received\n");
+                todo_wine ok(endpoint_reference_seen == TRUE, "EndpointReference not received\n");
+                todo_wine ok(app_sequence_seen == TRUE, "AppSequence not received\n");
+                todo_wine ok(metadata_version_seen == TRUE, "MetadataVersion not received\n");
+                todo_wine ok(message_ok == TRUE, "ProbeMatches message metadata not received\n");
+                todo_wine ok(any_header_seen == TRUE, "Custom header not received\n");
+                todo_wine ok(wine_ns_seen == TRUE, "Wine namespace not received\n");
+                todo_wine ok(body_probe_matches_seen == TRUE, "Body and Probe Matches elements not received\n");
+                todo_wine ok(any_body_seen == TRUE, "Custom body element not received\n");
+                todo_wine ok(types_seen == TRUE, "Types not received\n");
+
+after_matchprobe_test:
+                heap_free(publisherIdW);
+                heap_free(sequenceIdW);
+                heap_free(msg_storage);
+            }
+        }
     }
 
     SetEvent(probe_event);
@@ -611,23 +930,23 @@ static void Publish_tests(void)
     WSDXML_NAME header_any_name, another_name;
     WSDXML_NAMESPACE ns, ns2;
     WCHAR header_any_name_text[] = {'B','e','e','r',0};
-    WCHAR another_name_text[] = {'C','i','d','e','r',0};
     static const WCHAR header_any_text[] = {'P','u','b','l','i','s','h','T','e','s','t',0};
     static const WCHAR body_any_text[] = {'B','o','d','y','T','e','s','t',0};
     static const WCHAR endpoint_any_text[] = {'E','n','d','P','T','e','s','t',0};
     static const WCHAR ref_param_any_text[] = {'R','e','f','P','T','e','s','t',0};
     static const WCHAR uri[] = {'h','t','t','p',':','/','/','w','i','n','e','.','t','e','s','t','/',0};
     static const WCHAR prefix[] = {'w','i','n','e',0};
-    static const WCHAR uri2[] = {'h','t','t','p',':','/','/','m','o','r','e','.','t','e','s','t','s','/',0};
-    static const WCHAR prefix2[] = {'g','r','o','g',0};
     static const WCHAR uri3[] = {'h','t','t','p',':','/','/','t','h','i','r','d','.','u','r','l','/',0};
     WSD_NAME_LIST types_list;
     WSD_URI_LIST scopes_list, xaddrs_list;
+    unsigned char *probe_uuid_str;
 
     rc = WSDCreateDiscoveryPublisher(NULL, &publisher);
     ok(rc == S_OK, "WSDCreateDiscoveryPublisher(NULL, &publisher) failed: %08x\n", rc);
     ok(publisher != NULL, "WSDCreateDiscoveryPublisher(NULL, &publisher) failed: publisher == NULL\n");
 
+    publisher_instance = publisher;
+
     /* Test SetAddressFamily */
     rc = IWSDiscoveryPublisher_SetAddressFamily(publisher, 12345);
     ok(rc == E_INVALIDARG, "IWSDiscoveryPublisher_SetAddressFamily(12345) returned unexpected result: %08x\n", rc);
@@ -704,10 +1023,10 @@ static void Publish_tests(void)
     ok(rc == S_OK, "WSDXMLBuildAnyForSingleElement failed with %08x\n", rc);
 
     /* Create types list */
-    ns2.Uri = uri2;
-    ns2.PreferredPrefix = prefix2;
+    ns2.Uri = uri_more_tests;
+    ns2.PreferredPrefix = prefix_grog;
 
-    another_name.LocalName = another_name_text;
+    another_name.LocalName = (WCHAR *) name_cider;
     another_name.Space = &ns2;
 
     types_list.Next = malloc(sizeof(WSD_NAME_LIST));
@@ -721,10 +1040,10 @@ static void Publish_tests(void)
     scopes_list.Element = uri;
 
     scopes_list.Next->Next = NULL;
-    scopes_list.Next->Element = uri2;
+    scopes_list.Next->Element = uri_more_tests;
 
     xaddrs_list.Next = malloc(sizeof(WSD_URI_LIST));
-    xaddrs_list.Element = uri2;
+    xaddrs_list.Element = uri_more_tests;
 
     xaddrs_list.Next->Next = NULL;
     xaddrs_list.Next->Element = uri3;
@@ -818,8 +1137,23 @@ after_publish_test:
 
     /* Test the receiving of a probe message */
     probe_event = CreateEventW(NULL, TRUE, FALSE, NULL);
-    ok(send_udp_multicast_of_type(testProbeMessage, sizeof(testProbeMessage) - 1, AF_INET) == TRUE, "Sending Probe message failed\n");
-    todo_wine ok(WaitForSingleObject(probe_event, 2000) == WAIT_OBJECT_0, "Probe message not received\n");
+
+    UuidCreate(&probe_message_id);
+    UuidToStringA(&probe_message_id, &probe_uuid_str);
+
+    ok(probe_uuid_str != NULL, "Failed to create UUID for probe message\n");
+
+    if (probe_uuid_str != NULL)
+    {
+        char probe_message[sizeof(testProbeMessage) + 50];
+        sprintf(probe_message, testProbeMessage, probe_uuid_str);
+
+        ok(send_udp_multicast_of_type(probe_message, strlen(probe_message), AF_INET) == TRUE, "Sending Probe message failed\n");
+        ok(WaitForSingleObject(probe_event, 10000) == WAIT_OBJECT_0, "Probe message not received\n");
+
+        RpcStringFreeA(&probe_uuid_str);
+    }
+
     CloseHandle(probe_event);
 
     ref = IWSDiscoveryPublisher_Release(publisher);
